*************************************************************************************
*														*
*	Scrolling text rectangle demo for EASy68K V1.0		2011/05/10			*
*														*
*	The program starts by asking for the window top edge, left edge, width		*
*	and height. If the response gives a value out of range the query will		*
*	be repeated. The valid range is given in each query. To use the default		*
*	values either enter a value of zero or just hit [enter].				*
*														*
*	The screen will be cleared and the window border drawn then the text will	*
*	be output, a character at a time to the window. The delay is for visual		*
*	effect only, without it the text would just be a blur. Once the text is		*
*	going the program may be restarted by hitting [enter] or quit by hitting	*
*	[escape].												*
*														*
*	The text as it is output is left justified, no line starts with a space		*
*	character and an attenpt is made not to wrap words across the right edge	*
*	of the window. This will fail if any whole word is longer than the window	*
*	is wide.												*
*														*
*************************************************************************************
*
* some equates

ESC	EQU	$1B					* escape character
CR	EQU	$0D					* [CR] character


*************************************************************************************
*

	ORG		$1000

start
	BSR		Initialise			* go setup everything
restart
	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	BSR		get_window_top		* get the window top position
	BSR		get_window_left		* get the window left position
	BSR		get_window_height		* get the window height
	BSR		get_window_width		* get the window width

	BSR		draw_border			* draw the window border

* main loop

restart_main_loop
	LEA		demo_text(pc),a0		* set the pointer to the demo text
main_loop
	MOVE.b	(a0)+,d1			* get the next text chatracter
	BEQ.s		restart_main_loop		* if null reset the text pointer

	BSR		output_character		* go output the text

	MOVEQ		#7,d0				* read the key status
	TRAP		#15

	TST.b		d1				* test the result
	BEQ.s		end_main_loop		* if no key go do the next character

	MOVEQ		#5,d0				* read a key
	TRAP		#15

	CMPI.b	#CR,d1			* compare with [CR]
	BEQ.s		restart			* if [CR] go restart the program

	CMPI.b	#ESC,d1			* compare with [ESC]
	BNE.s		end_main_loop		* if not [ESC] go do the next character

	MOVE.w	#-1,quit(a3)		* flag quit
end_main_loop
	TST.w		quit(a3)			* test the quit flag
	BEQ.s		main_loop			* if not quit go get another key

* all done so clear up and say goodbye

	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	LEA		goodbye_message(pc),a1	* point to the goodbye message
	MOVEQ		#14,d0			* output text without [CR][LF]
	TRAP		#15

	MOVEQ		#9,d0				* else halt the simulator
	TRAP		#15


goodbye_message
	dc.b	$0D,$0A,'  Goodbye',$00

	ds.w	0					* ensure even


*************************************************************************************
*
* clear the screen and draw the scrolly window border

draw_border
	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	MOVE.w	window_row(a3),d7		* get the window top row
	SUBQ.w	#2,d7				* - 1 for loop and - 1 for offset

* output the blank lines

	MOVEQ		#$0A,d1			* set [LF]
	MOVEQ		#6,d0				* output character
line_lp
	TRAP		#15

	DBF		d7,line_lp			* loop if more to do

* calculate the string starts

	LEA		space_text(pc),a2		* point to the space text
	LEA		bars_text(pc),a4		* point to the bars string text
	LEA		hole_text(pc),a5		* point to the right hole string text
	MOVEA.l	a5,a6				* copy the pointer the to right hole text

	MOVEQ		#2,d0				* + 2 for offset
	SUB.w		window_col(a3),d0		* - the left column
	LEA		(a2,d0.w),a2		* offset to the requited string start

	SUBQ.w	#1,d0				* - 1 for hole left string
	LEA		(a6,d0.w),a6		* offset to the requited string start

	MOVEQ		#-3,d0			* - 3 for offset
	SUB.w		win_width(a3),d0		* - the width
	LEA		(a4,d0.w),a4		* offset to the requited string start
	LEA		(a5,d0.w),a5		* offset to the requited string start

* output the top border

	MOVEA.l	a2,a1				* copy the space string start
	MOVEQ		#14,d0			* output text without [CR][LF]
	TRAP		#15

	MOVEQ		#'+',d1			* set "+"
	MOVEQ		#6,d0				* output character
	TRAP		#15

	MOVEA.l	a4,a1				* copy the bars string start
	MOVEQ		#13,d0			* output text with [CR][LF]
	TRAP		#15

* top border done, now do the edges

	MOVE.w	win_height(a3),d7		* get the window height
	SUBQ.w	#1,d7				* - 1 for loop type
lines_lp
	MOVEA.l	a6,a1				* copy the left hole string start
	MOVEQ		#14,d0			* output text without [CR][LF]
	TRAP		#15

	MOVEA.l	a5,a1				* copy the right hole string start
	MOVEQ		#13,d0			* output text with [CR][LF]
	TRAP		#15

	DBF		d7,lines_lp			* loop if more to do

* edges done, now do the bottom border

	MOVEA.l	a2,a1				* copy the space string start
	MOVEQ		#14,d0			* output text without [CR][LF]
	TRAP		#15

	MOVEQ		#'+',d1			* set "+"
	MOVEQ		#6,d0				* output character
	TRAP		#15

	MOVEA.l	a4,a1				* copy the bars string start
	MOVEQ		#13,d0			* output text with [CR][LF]
	TRAP		#15

	BRA		position_cursor		* position the cursor and return

* text for making the borders

	dc.b	'--------------------------------------'
	dc.b	'------------------------------------+'
bars_text
	dc.b	$00

	dc.b	'                                      '
	dc.b	'                                    |'
hole_text
	dc.b	$00

	dc.b	'                                      '
	dc.b	'                           '
space_text
	dc.b	$00

	dc.w	0					* ensure even


*************************************************************************************
*
* output a character to the scrolly window, character is in d1.b

output_character
	CMPI.b	#$0D,d1			* compare the character with [CR]
	BEQ.s		position_cursor		* if [CR] go position the cursor

	CMPI.b	#$0A,d1			* compare the character with [LF]
	BEQ.s		was_lf			* if [LF] go handle it

	MOVE.w	cur_col(a3),d2		* get the cursor column

	CMPI.b	#' ',d1			* compare the character with [SPACE]
	BNE.s		wasnt_space			* if not [SPACE] go output the character

* don't put [SPACE] characters in the leftmost column, this makes sure all the text
* lines are left justified

	CMP.w		window_col(a3),d2		* compare the cursor with window left
	BEQ.s		output_done			* if space and first character skip it

* it was a space and not in the leftmost column so see if next bit of text will fit

	MOVEQ		#-1,d0			* set the index for the preincrement
find_space
	ADDQ.w	#1,d0				* increment the index to the next character
	CMP.b		(a0,d0.w),d1		* compare the next byte with [SPACE]
	BCS.s		find_space			* if not the next [SPACE], [CR] or [LF] go get
							* another character

* found the next space, check if the intervening text will fit

	ADD.w		d2,d0				* add the cursor column
	SUB.w		window_col(a3),d0		* - the window start column
	SUB.w		win_width(a3),d0		* - the window width
	BCC.s		was_lf			* if it doesn't fit go scroll the text

wasnt_space
	MOVEQ		#6,d0				* else output the character
	TRAP		#15

	ADDQ.w	#1,d2				* shift the cursor right
	MOVE.w	window_col(a3),d1		* get the window column
	ADD.w		win_width(a3),d1		* add the window width
	CMP.w		d2,d1				* compare the current column with the limit
	BNE.s		output_done			* if not there yet just exit

* at the right edge so scroll the window
*
* 25	scroll text rectangle
*	Pre: d1.l =	High 16 bits = Top row (0 to 128)
*			Low 16 bits = Left column (0 to 255)
*	     d2.l =	High 16 bits = Height in rows (Top + Height max 128)
*			Low 16 bits = Width in columns (Left + Width max 255)
*	     d3.w = 0 scroll up
*	     d3.w = 1 scroll down
*	     d3.w = 2 scroll left
*	     d3.w = 3 scroll right

was_lf
	MOVE.l	window_row(a3),d1		* get the top row/left column
	MOVE.l	win_height(a3),d2		* get the window height/width

	MOVEQ		#0,d3				* scroll up
	MOVEQ		#25,d0			* scroll text rectangle
	TRAP		#15

* position the cursor
*
* 11	set the cursor position:
*	The high byte of d1.w holds the column number	0 - 255
*	The low byte of d1.w holds the row number		0 - 128
*	0,0 is top left

position_cursor
	MOVE.w	window_col(a3),d2		* get the window left column
	MOVE.w	d2,d0				* copy the cursor column
	ASL.w		#8,d0				* shift it to the high byte

	MOVE.w	window_row(a3),d1		* get the window top row
	ADD.w		win_height(a3),d1		* add the window height to the row
	SUBQ.w	#1,d1				* - 1
	OR.w		d0,d1				* OR in the column
	MOVEQ		#11,d0			* position the cursor
	TRAP		#15

output_done
	MOVE.w	d2,cur_col(a3)		* save the cursor column
	MOVEQ		#10,d3			* get the inter unit timing
	BRA		wait_time			* go wait for the next d3.w*10 ms

 
*************************************************************************************
*
* setup stuff

Initialise
	LEA		variables(pc),a3		* get the variables base address

	MOVEQ		#0,d1				* echo off
	MOVEQ		#12,d0			* set keyboard echo
	TRAP		#15

	LEA		variables(pc),a3		* get the variables base address

	MOVE.w	d1,quit(a3)			* clear the quit flag

*##	BSR.s		set_timer			* setup the 10ms timer
*##	RTS


*************************************************************************************
*
* setup the interrupt timer

set_timer
	MOVE.w	#0,ms_count(a3)		* clear the 10 miliseconds counter

	LEA		timer_interrupt(pc),a0	* get the timer interrupt routine address
	MOVE.l	a0,$64.w			* save the timer interrupt as interrupt 1

	MOVEQ		#6,d1				* set auto IRQ
	MOVEQ		#$81,d2			* enable IRQ 1
	MOVEQ		#10,d3			* set the time in ms
	MOVEQ		#32,d0			* set simulator hardware
	TRAP		#15

	MOVEQ		#5,d1				* enable exceptions
	MOVEQ		#32,d0			* set simulator hardware
	TRAP		#15

	RTS


*************************************************************************************
*
* timer interrupt. the timer interrupt should be triggered every 10ms

timer_interrupt
	TST.w		ms_count(a3)		* test the 10 miliseconds counter
	BEQ.s		exit_interrupt		* if zero just exit

	SUBQ.w	#1,ms_count(a3)		* else decrement the 10 miliseconds counter
exit_interrupt
	RTE


*************************************************************************************
*
* wait for d3.w*10 milliseconds

wait_time
	MOVE.w	d3,ms_count(a3)		* set the 10 miliseconds counter
wait_loop
	TST.w		ms_count(a3)		* test the 10 miliseconds counter
	BNE.s		wait_loop			* loop if not done

	RTS


*************************************************************************************
*
* get the window top position

get_window_top
	LEA		window_top_null(pc),a1	* point to the window top null string
	MOVEQ		#13,d0			* display a string witht [CR][LF]
	TRAP		#15

get_win_top
	LEA		window_top_msg(pc),a1	* point to the window top query string
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	BSR		getnumber			* get a number in d2.w with echo to the screen
	MOVEQ		#4,d3				* set the default top position

	TST.w		d2				* test if the user just hit enter
	BEQ.s		use_default_top		* if so go use the default value

	CMPI.w	#2,d2				* compare the window top with the minimum
							* allowed
	BMI.s		get_win_top			* if less than the minimum go try again

	CMPI.w	#20,d2			* compare the window top with the maximum
							* allowed
	BHI.s		get_win_top			* if greater than the maximum go try again

	MOVE.w	d2,d3				* overwrite the default value
use_default_top
	MOVE.w	d3,window_row(a3)		* save the new window top
	RTS

window_top_msg
	dc.b	'  Enter the window top row     [2 - 20]  : '
window_top_null
	dc.b	$00

	ds.w	0					* ensure even


*************************************************************************************
*
* get the window left position

get_window_left
	LEA		window_left_msg(pc),a1	* point to the window left query string
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	BSR		getnumber			* get a number in d2.w with echo to the screen
	MOVEQ		#8,d3				* set the default left position

	TST.w		d2				* test if the user just hit enter
	BEQ.s		use_default_left		* if so go use the default left value

	CMPI.w	#4,d2				* compare the window left with the minimum
							* allowed
	BMI.s		get_window_left		* if less than the minimum go try again

	CMPI.w	#44,d2			* compare the window left with the maximum
							* allowed
	BHI.s		get_window_left		* if greater than the maximum go try again

	MOVE.w	d2,d3				* overwrite the default value
use_default_left
	MOVE.w	d3,window_col(a3)		* save the new window left
	RTS

window_left_msg
	dc.b	'  Enter the window left column [4 - 44]  : '
	dc.b	$00

	ds.w	0					* ensure even


*************************************************************************************
*
* get the window height

get_window_height
	LEA		window_height_msg(pc),a1
							* point to the window height query string
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	MOVEQ		#30,d1			* set the maximum allowed
	SUB.w		window_row(a3),d1		* minus the window top row
	MOVE.w	d1,d7				* copy the maximum allowed
	MOVEQ		#3,d0				* display d1.l in decimal
	TRAP		#15

	LEA		window_height_suffix(pc),a1
							* point to the window height query suffix
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	BSR		getnumber			* get a number in d2.w with echo to the screen
	MOVEQ		#8,d3				* set the default height

	TST.w		d2				* test if the user just hit enter
	BEQ.s		use_default_height	* if so go use the default height value

	CMPI.w	#2,d2				* compare the window height with the minimum
							* allowed
	BMI.s		get_window_height		* if less than the minimum go try again

	CMP.w		d7,d2				* compare the window height with the maximum
							* allowed
	BHI.s		get_window_height		* if greater than the maximum go try again

	MOVE.w	d2,d3				* overwrite the default value
use_default_height
	MOVE.w	d3,win_height(a3)		* save the new window height
	RTS

window_height_msg
	dc.b	'  Enter the window height      [2 - '
	dc.b	$00

window_height_suffix
	dc.b	']  : '
	dc.b	$00

	ds.w	0					* ensure even


*************************************************************************************
*
* get the window width

get_window_width
	LEA		window_width_msg(pc),a1	* point to the window width query string
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	MOVEQ		#76,d1			* set the maximum allowed
	SUB.w		window_col(a3),d1		* minus the window left column
	MOVE.w	d1,d7				* copy the maximum allowed
	MOVEQ		#3,d0				* display d1.l in decimal
	TRAP		#15

	LEA		window_width_suffix(pc),a1
							* point to the window width query suffix
	MOVEQ		#14,d0			* display a string without [CR][LF]
	TRAP		#15

	BSR		getnumber			* get a number in d2.w with echo to the screen
	MOVEQ		#32,d3			* set the default width

	TST.w		d2				* test if the user just hit enter
	BEQ.s		use_default_width		* if so go use the default height value

	CMPI.w	#9,d2				* compare the window width with the minimum
							* allowed
	BMI.s		get_window_width		* if less than the minimum go try again

	CMP.w		d7,d2				* compare the window width with the maximum
							* allowed
	BHI.s		get_window_width		* if greater than the maximum go try again

	MOVE.w	d2,d3				* overwrite the default value
use_default_width
	MOVE.w	d3,win_width(a3)		* save the new window height
	RTS

window_width_msg
	dc.b	'  Enter the window width       [9 - '
	dc.b	$00

window_width_suffix
	dc.b	']  : '
	dc.b	$00

	ds.w	0					* ensure even


*************************************************************************************
*
* get a number in d2.w with echo to the screen

getnumber
	MOVEQ		#-1,d1			* echo on
	MOVEQ		#12,d0			* set keyboard echo
	TRAP		#15

	MOVEQ		#4,d0				* read a value into d1.l
	TRAP		#15

	MOVE.w	d1,d2				* copy the number

	MOVEQ		#0,d1				* echo off
	MOVEQ		#12,d0			* set keyboard echo
	TRAP		#15

	RTS


*************************************************************************************
*
* the scrolly text

demo_text
	dc.b	'Lorem ipsum dolor sit amet, consectetur adipisic'
	dc.b	'ing elit, sed do eiusmod tempor incididunt ut la'
	dc.b	'bore et dolore magna aliqua. Ut enim ad minim ve'
	dc.b	'niam, quis nostrud exercitation ullamco laboris '
	dc.b	'nisi ut aliquip ex ea commodo consequat. Duis au'
	dc.b	'te irure dolor in reprehenderit in voluptate vel'
	dc.b	'it esse cillum dolore eu fugiat nulla pariatur. '
	dc.b	'Excepteur sint occaecat cupidatat non proident, '
	dc.b	'sunt in culpa qui officia deserunt mollit anim i'
	dc.b	'd est laborum.',$0A,$0A

	dc.b	'Sed ut perspiciatis unde omnis iste natus error s'
	dc.b	'it voluptatem accusantium doloremque laudantium, '
	dc.b	'totam rem aperiam, eaque ipsa quae ab illo invent'
	dc.b	'ore veritatis et quasi architecto beatae vitae di'
	dc.b	'cta sunt explicabo. Nemo enim ipsam voluptatem qu'
	dc.b	'ia voluptas sit aspernatur aut odit aut fugit, se'
	dc.b	'd quia consequuntur magni dolores eos qui ratione'
	dc.b	' voluptatem sequi nesciunt. Neque porro quisquam '
	dc.b	'est, qui dolorem ipsum quia dolor sit amet, conse'
	dc.b	'ctetur, adipisci velit, sed quia non numquam eius'
	dc.b	' modi tempora incidunt ut labore et dolore magnam'
	dc.b	' aliquam quaerat voluptatem. Ut enim ad minima ve'
	dc.b	'niam, quis nostrum exercitationem ullam corporis '
	dc.b	'suscipit laboriosam, nisi ut aliquid ex ea commod'
	dc.b	'i consequatur? Quis autem vel eum iure reprehende'
	dc.b	'rit qui in ea voluptate velit esse quam nihil mol'
	dc.b	'estiae consequatur, vel illum qui dolorem eum fug'
	dc.b	'iat quo voluptas nulla pariatur?',$0A,$0A

	dc.b	'But I must explain to you how all this mistaken i'
	dc.b	'dea of denouncing pleasure and praising pain was '
	dc.b	'born and I will give you a complete account of th'
	dc.b	'e system, and expound the actual teachings of the'
	dc.b	' great explorer of the truth, the master-builder '
	dc.b	'of human happiness. No one rejects, dislikes, or '
	dc.b	'avoids pleasure itself, because it is pleasure, b'
	dc.b	'ut because those who do not know how to pursue pl'
	dc.b	'easure rationally encounter consequences that are'
	dc.b	' extremely painful. Nor again is there anyone who'
	dc.b	' loves or pursues or desires to obtain pain of it'
	dc.b	'self, because it is pain, but because occasionall'
	dc.b	'y circumstances occur in which toil and pain can '
	dc.b	'procure him some great pleasure. To take a trivia'
	dc.b	'l example, which of us ever undertakes laborious '
	dc.b	'physical exercise, except to obtain some advantag'
	dc.b	'e from it? But who has any right to find fault wi'
	dc.b	'th a man who chooses to enjoy a pleasure that has'
	dc.b	' no annoying consequences, or one who avoids a pa'
	dc.b	'in that produces no resultant pleasure?',$0A,$0A

	dc.b	'At vero eos et accusamus et iusto odio dignissimo'
	dc.b	's ducimus qui blanditiis praesentium voluptatum d'
	dc.b	'eleniti atque corrupti quos dolores et quas moles'
	dc.b	'tias excepturi sint occaecati cupiditate non prov'
	dc.b	'ident, similique sunt in culpa qui officia deseru'
	dc.b	'nt mollitia animi, id est laborum et dolorum fuga'
	dc.b	'. Et harum quidem rerum facilis est et expedita d'
	dc.b	'istinctio. Nam libero tempore, cum soluta nobis e'
	dc.b	'st eligendi optio cumque nihil impedit quo minus '
	dc.b	'id quod maxime placeat facere possimus, omnis vol'
	dc.b	'uptas assumenda est, omnis dolor repellendus. Tem'
	dc.b	'poribus autem quibusdam et aut officiis debitis a'
	dc.b	'ut rerum necessitatibus saepe eveniet ut et volup'
	dc.b	'tates repudiandae sint et molestiae non recusanda'
	dc.b	'e. Itaque earum rerum hic tenetur a sapiente dele'
	dc.b	'ctus, ut aut reiciendis voluptatibus maiores alia'
	dc.b	's consequatur aut perferendis doloribus asperiore'
	dc.b	's repellat.',$0A,$0A

	dc.b	'On the other hand, we denounce with righteous ind'
	dc.b	'ignation and dislike men who are so beguiled and '
	dc.b	'demoralized by the charms of pleasure of the mome'
	dc.b	'nt, so blinded by desire, that they cannot forese'
	dc.b	'e the pain and trouble that are bound to ensue; a'
	dc.b	'nd equal blame belongs to those who fail in their'
	dc.b	' duty through weakness of will, which is the same'
	dc.b	' as saying through shrinking from toil and pain. '
	dc.b	'These cases are perfectly simple and easy to dist'
	dc.b	'inguish. In a free hour, when our power of choice'
	dc.b	' is untrammelled and when nothing prevents our be'
	dc.b	'ing able to do what we like best, every pleasure '
	dc.b	'is to be welcomed and every pain avoided. But in '
	dc.b	'certain circumstances and owing to the claims of '
	dc.b	'duty or the obligations of business it will frequ'
	dc.b	'ently occur that pleasures have to be repudiated '
	dc.b	'and annoyances accepted. The wise man therefore a'
	dc.b	'lways holds in these matters to this principle of'
	dc.b	' selection: he rejects pleasures to secure other '
	dc.b	'greater pleasures, or else he endures pains to av'
	dc.b	'oid worse pains.',$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A
	dc.b	$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A,$0A
	dc.b	$0A,$0A,$0A,$0A,$0A,$0A,$0A,$00

	ds.w	0					* ensure even


*************************************************************************************
*
* variables

variables

	OFFSET	0				* going to use relative addressing

ms_count
	ds.w	1					* 10 milliiseconds count down timer

window_row
	ds.w	1					* window top row
window_col
	ds.w	1					* window left column
win_height
	ds.w	1					* window height in rows
win_width
	ds.w	1					* window width in columns
cur_col
	ds.w	1					* cursor column

quit
	ds.w	1					* quit flag


*************************************************************************************

	END		start

*************************************************************************************
